# Generated by roxygen2: do not edit by hand

S3method(AIC,mlmodel)
S3method(AIC,summary.mlmodel)
S3method(BIC,mlmodel)
S3method(BIC,summary.mlmodel)
S3method(GOFtest,mlmodel)
S3method(IMtest,mlmodel)
S3method(coef,mlmodel)
S3method(confint,mlmodel)
S3method(find_predictors,mlmodel)
S3method(fitted,mlmodel)
S3method(fitted,values.mlmodel)
S3method(formula,mlmodel)
S3method(get_coef,mlmodel)
S3method(get_data,mlmodel)
S3method(get_predict,mlmodel)
S3method(get_vcov,mlmodel)
S3method(gradientObs,mlmodel)
S3method(hessianObs,mlmodel)
S3method(logLik,mlmodel)
S3method(logLik,summary.mlmodel)
S3method(loglikeObs,mlmodel)
S3method(lrtest,mlmodel)
S3method(nobs,mlmodel)
S3method(predict,ml_beta)
S3method(predict,ml_gamma)
S3method(predict,ml_lm)
S3method(predict,ml_logit)
S3method(predict,ml_negbin)
S3method(predict,ml_poisson)
S3method(predict,ml_probit)
S3method(predict,mlmodel)
S3method(print,GOFtest.mlmodel)
S3method(print,IMtest.mlmodel)
S3method(print,OVDtest)
S3method(print,lrtest.mlmodel)
S3method(print,summary.ml_beta)
S3method(print,summary.ml_gamma)
S3method(print,summary.ml_lm)
S3method(print,summary.ml_logit)
S3method(print,summary.ml_negbin)
S3method(print,summary.ml_poisson)
S3method(print,summary.ml_probit)
S3method(print,vuongtest.mlmodel)
S3method(print,waldtest.mlmodel)
S3method(residuals,mlmodel)
S3method(se,mlmodel)
S3method(set_coef,mlmodel)
S3method(summary,ml_beta)
S3method(summary,ml_gamma)
S3method(summary,ml_lm)
S3method(summary,ml_logit)
S3method(summary,ml_negbin)
S3method(summary,ml_poisson)
S3method(summary,ml_probit)
S3method(summary,mlmodel)
S3method(update,ml_poisson)
S3method(update,mlmodel)
S3method(vcov,mlmodel)
S3method(vuongtest,mlmodel)
S3method(waldtest,mlmodel)
export("%||%")
export(GOFtest)
export(IMtest)
export(OVDtest)
export(find_variables.mlmodel)
export(get_modeldata.mlmodel)
export(gradientObs)
export(hessianObs)
export(loglikeObs)
export(lrtest)
export(ml_beta)
export(ml_gamma)
export(ml_lm)
export(ml_logit)
export(ml_negbin)
export(ml_poisson)
export(ml_probit)
export(se)
export(vuongtest)
export(waldtest)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_warn)
importFrom(hardhat,default_formula_blueprint)
importFrom(hardhat,forge)
importFrom(hardhat,mold)
importFrom(insight,find_predictors)
importFrom(insight,find_variables)
importFrom(insight,get_data)
importFrom(marginaleffects,get_coef)
importFrom(marginaleffects,get_predict)
importFrom(marginaleffects,get_vcov)
importFrom(marginaleffects,set_coef)
importFrom(maxLik,maxLik)
importFrom(rlang,"%||%")
importFrom(rlang,arg_match)
importFrom(rlang,enquo)
importFrom(rlang,eval_tidy)
importFrom(stats,.lm.fit)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,cov2cor)
importFrom(stats,dlogis)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,fitted)
importFrom(stats,fitted.values)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,plogis)
importFrom(stats,pnbinom)
importFrom(stats,pnorm)
importFrom(stats,ppois)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(tibble,tibble)
importFrom(utils,capture.output)
