% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/beta-postestimation.R,
%   R/gamma-postestimation.R, R/lm-postestimation.R, R/logit-postestimation.R,
%   R/mlmodel-utils.R, R/negbin-postestimation.R, R/poisson-postestimation.R,
%   R/probit-postestimation.R
\name{summary.ml_beta}
\alias{summary.ml_beta}
\alias{summary.ml_gamma}
\alias{summary.ml_lm}
\alias{summary.ml_logit}
\alias{summary.mlmodel}
\alias{summary.ml_negbin}
\alias{summary.ml_poisson}
\alias{summary.ml_probit}
\title{Summary for mlmodel objects}
\usage{
\method{summary}{ml_beta}(
  object,
  correlation = FALSE,
  vcov = NULL,
  vcov.type = "oim",
  cl_var = NULL,
  repetitions = 999,
  seed = NULL,
  progress = FALSE,
  ...
)

\method{summary}{ml_gamma}(
  object,
  correlation = FALSE,
  vcov = NULL,
  vcov.type = "oim",
  cl_var = NULL,
  repetitions = 999,
  seed = NULL,
  progress = FALSE,
  ...
)

\method{summary}{ml_lm}(
  object,
  correlation = FALSE,
  vcov = NULL,
  vcov.type = "oim",
  cl_var = NULL,
  repetitions = 999,
  seed = NULL,
  progress = FALSE,
  ...
)

\method{summary}{ml_logit}(
  object,
  correlation = FALSE,
  vcov = NULL,
  vcov.type = "oim",
  cl_var = NULL,
  repetitions = 999,
  seed = NULL,
  progress = FALSE,
  ...
)

\method{summary}{mlmodel}(
  object,
  correlation = FALSE,
  vcov = NULL,
  vcov.type = "oim",
  cl_var = NULL,
  repetitions = 999,
  seed = NULL,
  progress = FALSE,
  ...
)

\method{summary}{ml_negbin}(
  object,
  correlation = FALSE,
  vcov = NULL,
  vcov.type = "oim",
  cl_var = NULL,
  repetitions = 999,
  seed = NULL,
  progress = FALSE,
  ...
)

\method{summary}{ml_poisson}(
  object,
  correlation = FALSE,
  vcov = NULL,
  vcov.type = "oim",
  cl_var = NULL,
  repetitions = 999,
  seed = NULL,
  progress = FALSE,
  ...
)

\method{summary}{ml_probit}(
  object,
  correlation = FALSE,
  vcov = NULL,
  vcov.type = "oim",
  cl_var = NULL,
  repetitions = 999,
  seed = NULL,
  progress = FALSE,
  ...
)
}
\arguments{
\item{object}{A fitted model object of class \code{"mlmodel"}.}

\item{correlation}{Logical. Should the correlation matrix of the estimated
parameters be included in the output? Default is \code{FALSE}. If \code{TRUE} the
correlation matrix will be computed, and stored in the \code{'summary.mlmodel'}
object the function returns.}

\item{vcov}{Optional user-supplied variance-covariance matrix. If provided,
it will be used instead of computing one internally.}

\item{vcov.type}{Character string specifying the type of variance-covariance
matrix to use. See \link[=vcov.mlmodel]{vcov}.}

\item{cl_var}{Character string or vector. Name of the clustering variable
or the vector itself. See \link[=vcov.mlmodel]{vcov}.}

\item{repetitions}{Integer. Number of bootstrap replications when
\code{vcov.type = "boot"}. Default is 999.}

\item{seed}{Integer. Random seed for reproducibility when \code{vcov.type = "boot"}.
If \code{NULL}, a random seed is generated.}

\item{progress}{Logical. Should a progress bar be displayed during
bootstrapping or jackknifing? Default is \code{FALSE} (silent).}

\item{...}{Further arguments passed to methods.}
}
\value{
An object of class \code{summary.mlmodel} and any other class that extends
it.
}
\description{
Summary for mlmodel objects
}
\details{
Coefficient names in the fitted object use the prefixes \verb{value::} and
\verb{scale::} to identify to which equation they belong to, and to avoid
confusion when the same variable(s) appear(s) in both the value and scale
equations.
}
\examples{

data(mroz)
mroz$incthou <- mroz$faminc / 1000

fit <- ml_lm(incthou ~ age + I(age^2) + huswage + educ + unem, 
             data = mroz)

# Default: observed information matrix
summary(fit)

# Different variance types
summary(fit, vcov.type = "opg")                    # Outer product of gradients
summary(fit, vcov.type = "robust")                 # Robust/sandwich estimator

# Clustered robust standard errors
summary(fit, vcov.type = "robust", cl_var = "age")

# Using a pre-computed variance matrix (e.g. bootstrap)
v_boot <- vcov(fit, type = "boot", repetitions = 100, seed = 123)
summary(fit, vcov = v_boot)

}
\author{
Alfonso Sanchez-Penalver
}
